package com.sun.xml.internal.bind.api.impl;

import java.util.List;
import java.util.StringTokenizer;
import java.util.ArrayList;
import javax.lang.model.SourceVersion;

public interface NameConverter
{
    public static final NameConverter standard = new NameConverter.Standard();
    public static final NameConverter jaxrpcCompatible = new NameConverter.Standard() {
        @Override
        protected boolean isPunct(char c) {
            return c == '.' || c == '-' || c == ';' || c == '·' || c == '·' || c == '\u06dd' || c == '۞';
        }
        
        @Override
        protected boolean isLetter(char c) {
            return super.isLetter(c) || c == '_';
        }
        
        @Override
        protected int classify(char c0) {
            if (c0 == '_') {
                return 2;
            }
            return super.classify(c0);
        }
    };
    public static final NameConverter smart = new NameConverter.Standard() {
        @Override
        public String toConstantName(String token) {
            String name = super.toConstantName(token);
            if (!SourceVersion.isKeyword(name)) {
                return name;
            }
            return 95 + name;
        }
    };
    
    public abstract String toClassName(String p0);
    
    public abstract String toInterfaceName(String p0);
    
    public abstract String toPropertyName(String p0);
    
    public abstract String toConstantName(String p0);
    
    public abstract String toVariableName(String p0);
    
    public abstract String toPackageName(String p0);
    
    public static class Standard extends NameUtil implements NameConverter
    {
        @Override
        public String toClassName(String s) {
            return this.toMixedCaseName(this.toWordList(s), true);
        }
        
        @Override
        public String toVariableName(String s) {
            return this.toMixedCaseName(this.toWordList(s), false);
        }
        
        @Override
        public String toInterfaceName(String token) {
            return this.toClassName(token);
        }
        
        @Override
        public String toPropertyName(String s) {
            String prop = this.toClassName(s);
            if (prop.equals("Class")) {
                prop = "Clazz";
                return prop;
            }
            return prop;
        }
        
        @Override
        public String toConstantName(String token) {
            return super.toConstantName(token);
        }
        
        @Override
        public String toPackageName(String nsUri) {
            int idx = nsUri.indexOf(58);
            String scheme = "";
            if (idx >= 0) {
                scheme = nsUri.substring(0, idx);
                if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("urn")) {
                    nsUri = nsUri.substring(idx + 1);
                }
            }
            ArrayList<String> tokens = tokenize(nsUri, "/: ");
            if (tokens.size() == 0) {
                return null;
            }
            if (tokens.size() > 1) {
                String lastToken = (String)tokens.get(tokens.size() - 1);
                idx = lastToken.lastIndexOf(46);
                if (idx > 0) {
                    lastToken = lastToken.substring(0, idx);
                    tokens.set(tokens.size() - 1, lastToken);
                }
            }
            String domain = (String)tokens.get(0);
            idx = domain.indexOf(58);
            if (idx >= 0) {
                domain = domain.substring(0, idx);
            }
            ArrayList<String> r = reverse(tokenize(domain, scheme.equals("urn") ? ".-" : "."));
            if (((String)r.get(r.size() - 1)).equalsIgnoreCase("www")) {
                r.remove(r.size() - 1);
            }
            tokens.addAll(1, r);
            tokens.remove(0);
            for (int i = 0; i < tokens.size(); ++i) {
                String token = (String)tokens.get(i);
                token = removeIllegalIdentifierChars(token);
                if (SourceVersion.isKeyword(token.toLowerCase())) {
                    token = 95 + token;
                }
                tokens.set(i, token.toLowerCase());
            }
            return combine(tokens, '.');
        }
        
        private static String removeIllegalIdentifierChars(String token) {
            StringBuilder newToken = new StringBuilder(token.length() + 1);
            for (int i = 0; i < token.length(); ++i) {
                char c = token.charAt(i);
                if (i == 0 && !Character.isJavaIdentifierStart(c)) {
                    newToken.append('_');
                }
                if (!Character.isJavaIdentifierPart(c)) {
                    newToken.append('_');
                }
                else {
                    newToken.append(c);
                }
            }
            return newToken.toString();
        }
        
        private static ArrayList<String> tokenize(String str, String sep) {
            StringTokenizer tokens = new StringTokenizer(str, sep);
            ArrayList<String> r = new ArrayList();
            while (tokens.hasMoreTokens()) {
                r.add(tokens.nextToken());
            }
            return r;
        }
        
        private static <T> ArrayList<T> reverse(List<T> a) {
            ArrayList<T> r = new ArrayList();
            for (int i = a.size() - 1; i >= 0; --i) {
                r.add(a.get(i));
            }
            return r;
        }
        
        private static String combine(List r, char sep) {
            StringBuilder buf = new StringBuilder(r.get(0).toString());
            for (int i = 1; i < r.size(); ++i) {
                buf.append(sep);
                buf.append(r.get(i));
            }
            return buf.toString();
        }
    }
}
